package ld26.hernblog.com;
import nme.display.Sprite;
import nme.events.MouseEvent;
import nme.geom.Rectangle;
import nme.media.Sound;
import nme.media.SoundChannel;
import nme.media.SoundTransform;

/**
 * ...
 * @author Ian Hern
 */

class Box extends Sprite
{		
	
	// max possible is 255
	private static var colourMagnitude:Float = 220.0;
	public static var MaxAnxiousness:Float = 4.0;
	
	private var inhaleSound:Sound;
	private var exhaleSound:Sound;
	private var humSound:Sound;
	private var humChannel:SoundChannel;
	private var humTransform:SoundTransform;
	
	private var bounds:Rectangle;
	
	// amount that the scale can go above 1
	private static var MaxBreatingScaleBonus:Float = 0.2;
	private var currentBreathingScaleBonus:Float;
	private var isBreathingIn:Bool = true;
	
	// this is used for the slide in when the box spawns
	private var velocityY:Float = 30.0;
	
	// from 0 to 6 (wraps around), represents the colour in simpler terms for use in calculation anxiousness
	public var colourScale:Float;
	public var myWidth:Int;
	public var myHeight:Int;
	
	private var anxiousness:Float = 0.0;
	public function getAnxiousness():Float
	{
		return anxiousness;
	}
	public function setAnxiousness(anxiousness:Float):Void 
	{
		if(Std.int(Math.floor(this.anxiousness * 100)) != Std.int(Math.floor(anxiousness * 100))){
			this.anxiousness = anxiousness;
			humTransform.volume = anxiousness / 8;
			if (humChannel != null){
				humChannel.soundTransform = humTransform;
			}
			changeColour(GetColourFromScale(colourScale, ((MaxAnxiousness + 0.5) - anxiousness) / (MaxAnxiousness + 0.5)));
		}
	}
	
	public function new(width:Int, height:Int, colourScale:Float, x:Int, y:Int, bounds:Rectangle, humSound:Sound, inhaleSound:Sound, exhaleSound:Sound) 
	{
		super();
		
		this.colourScale = colourScale;
		
		this.humSound = humSound;
		humTransform = new SoundTransform(0.01, 0);
		humChannel = humSound.play(0, 9999999, humTransform);
		
		this.inhaleSound = inhaleSound;
		this.exhaleSound = exhaleSound;
		
		myWidth = width;
		myHeight = height;
		
		changeColour(GetColourFromScale(colourScale, 1.0));
		
		this.x = x;
		this.y = y;
		this.bounds = bounds;
		
		this.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
		this.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		
		currentBreathingScaleBonus = MaxBreatingScaleBonus;
	}
	
	public function removeDragging() {
		this.stopDrag();
		this.removeEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
		this.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
	}
	
	public function update():Void 
	{
		simulateBreathing();
		
		this.y += velocityY;
		
		// apply friction
		velocityY *= 0.8;
		if (velocityY < 0.1)
		{
			velocityY = 0.0;
		}
	}
	
	public function unload():Void 
	{
		if (humChannel != null)
		{
			humChannel.stop();
		}
	}
	
	private function changeColour(colour:Int):Void
	{
		this.graphics.beginFill(colour);
		this.graphics.drawRect( -myWidth / 2, -myHeight / 2, myWidth, myHeight);
		this.graphics.endFill();
	}
	
	private function simulateBreathing():Void 
	{
		var breathingRate:Float = (MaxAnxiousness - anxiousness) * 0.08 + 0.65;
		currentBreathingScaleBonus *= breathingRate;
		
		var scale:Float = 1.0;
		if (isBreathingIn)
		{
			scale += MaxBreatingScaleBonus - currentBreathingScaleBonus;
		}
		else
		{
			scale += currentBreathingScaleBonus;
		}
		this.scaleX = scale;
		this.scaleY = scale;
		
		if (currentBreathingScaleBonus <= 0.007)
		{
			isBreathingIn = !isBreathingIn;
			currentBreathingScaleBonus = MaxBreatingScaleBonus;
		}
	}
	
	// return a number from 0 to 1 where 0 is square, and 1 is least square possible
	public function UnSquareness():Float
	{
		return Math.abs(myWidth - myHeight) / (BoxFactory.MaxSize - BoxFactory.MinSize);
	}
	
	// area of the shape scaled to be 0 to 1 based on possible sizes
	public function AreaFactor():Float
	{
		var minimumArea = Math.pow(BoxFactory.MinSize, 2);
		return (myWidth * myHeight - minimumArea) / (Math.pow(BoxFactory.MaxSize, 2) - minimumArea);
	}
	
	private function onMouseDown(e:MouseEvent):Void 
	{
		e.target.startDrag(false, bounds);
		if(inhaleSound != null){
			inhaleSound.play();
		}
	}
	
	private function onMouseUp(e:MouseEvent):Void 
	{
		e.target.stopDrag();
		if(exhaleSound != null){
			exhaleSound.play();
		}
	}
	
	// scale must be a number from 0 to 6
	// brightness scale is from 0 to 1, if its 0 then we should return black
	private function GetColourFromScale(scale:Float, brightnessScale:Float):Int
	{
		// portions are in the scale of 0 to 1
		var redPortion:Float;
		var greenPortion:Float;
		var bluePortion:Float;
		
		if (scale < 1.0)
		{
			redPortion = 1;
			greenPortion = scale;
			bluePortion = 0;
		}
		else if (scale < 2.0)
		{
			redPortion = 1.0 - (scale % 1.0);
			greenPortion = 1;
			bluePortion = 0;
		}
		else if (scale < 3.0)
		{
			redPortion = 0;
			greenPortion = 1;
			bluePortion = scale % 1.0;
		}
		else if (scale < 4.0)
		{
			redPortion = 0;
			greenPortion = 1.0 - (scale % 1.0);
			bluePortion = 1;
		}
		else if (scale < 5.0)
		{
			redPortion = scale % 1.0;
			greenPortion = 0;
			bluePortion = 1;
		}
		else
		{
			redPortion = 1;
			greenPortion = 0;
			bluePortion = 1.0 - (scale % 1.0);
		}
				
		return Std.int(Math.floor(redPortion * colourMagnitude * brightnessScale) * 256 * 256 + Math.floor(greenPortion * colourMagnitude * brightnessScale) * 256 + bluePortion * colourMagnitude * brightnessScale);
	}
}