package ld26.hernblog.com;

import browser.display.Bitmap;
import nme.Assets;
import nme.display.Sprite;
import nme.events.Event;
import nme.events.MouseEvent;
import nme.geom.Rectangle;
import nme.Lib;
import nme.media.Sound;
import nme.text.TextField;
import nme.text.TextFormat;
import nme.utils.Timer;
import nme.events.TimerEvent;
import nme.display.Bitmap;

/**
 * ...
 * @author Ian Hern
 */
 
enum GameState {
	Playing;
	GameOverAnimation;
	GameOver;
}
 
class Main extends Sprite 
{
	
	public function new() 
	{
		super();
		#if iphone
		Lib.current.stage.addEventListener(Event.RESIZE, init);
		#else
		addEventListener(Event.ADDED_TO_STAGE, init);
		#end
	}
	
	private var boxSpawnTimer:Timer;
	private var gameUpdateTimer:Timer;
	
	private var boxFactory:BoxFactory;
	
	private var boxes:Array<Box>;
	
	// these weights determine how anxious the boxes get when near other boxes that differ in the type specified
	private var degreeOfSquarenessWeight:Float = 1.0;
	private var areaWeight:Float = 1.0;
	private var colourWeight:Float = 1.0;
	
	private static var MaxAnxiousnessDistance:Float = 250.0;
	
	private var boxSlideSound:Sound;
	private var popSound:Sound;
	
	private var gameState:GameState;
	private var gameLossX:Float;
	private var gameLossY:Float;
	private var gameLossDiameter:Float;
	private static var gameLossDiameterGrowthRate:Float = 20;
	
	private var score:Int = 0;
	//private var gameOverTextField:TextField;
	private var scoreTextField:TextField;
	
	private var title:Bitmap;
	
	private function init(e) 
	{
		gameState = GameOver;
		this.stage.color = 0xFFFFFF;
				
		var spriteClickArea:Sprite = new Sprite();
		spriteClickArea.graphics.beginFill(0xFFFFFF);
		spriteClickArea.graphics.drawRect(0, 0, 800, 480);
		spriteClickArea.graphics.endFill();
		addChild(spriteClickArea);
		
		title = new Bitmap(Assets.getBitmapData("img/title.png"));
		addChild(title);
		
		var font = Assets.getFont("assets/font/Existence-Light-webfont.ttf");
		var format = new TextFormat(font.fontName, 30, 0x303030);
		//gameOverTextField = new TextField();
		//gameOverTextField.defaultTextFormat = format;
		//gameOverTextField.selectable = false;
		//gameOverTextField.embedFonts = true;
		//gameOverTextField.width = 400;
		//gameOverTextField.height = 120;
		//gameOverTextField.x = 280;
		//gameOverTextField.y = 160;
		
		scoreTextField = new TextField();
		scoreTextField.defaultTextFormat = format;
		scoreTextField.selectable = false;
		scoreTextField.embedFonts = true;
		scoreTextField.width = 120;
		scoreTextField.height = 40;
		scoreTextField.x = 10;
		scoreTextField.y = 10;
		scoreTextField.text = "0";
		addChild(scoreTextField);
		
		boxSlideSound = Assets.getSound("assets/sound/eggCartonSlide.wav");
		popSound = Assets.getSound("assets/sound/pop.wav");
		
		boxes = new Array<Box>();		
		boxFactory = new BoxFactory(this.stage.stageWidth, this.stage.stageHeight);
		
		randomizeWeights();
		
		boxSpawnTimer = new Timer(6000);
		boxSpawnTimer.addEventListener(TimerEvent.TIMER, spawnBox);
		boxSpawnTimer.start();
		
		gameUpdateTimer = new Timer(33);
		gameUpdateTimer.addEventListener(TimerEvent.TIMER, gameUpdate);
		gameUpdateTimer.start();
		
		spriteClickArea.addEventListener(MouseEvent.CLICK, onMouseClick);
	}
	
	private function randomizeWeights():Void 
	{	
		var defaultWeight:Float = 1.0;
		degreeOfSquarenessWeight = defaultWeight;
		areaWeight = defaultWeight;
		colourWeight = defaultWeight;
		
		var random = Math.random() * 3;
		if (random < 1) {
			degreeOfSquarenessWeight = 0;
		}
		else if (random < 2) {
			areaWeight = 0;
		}
		else {
			colourWeight = 0;
		}
	}
	
	private function gameUpdate(e:TimerEvent)
	{
		if (gameState == Playing)
		{
			increaseAnxiousness();
			
			for (box in boxes)
			{
				if (box.getAnxiousness() > Box.MaxAnxiousness)
				{
					gameState = GameOverAnimation;
					gameLossDiameter = 0;
					gameLossX = box.x;
					gameLossY = box.y;
				}
			}
		}
		else if (gameState == GameOverAnimation)
		{
			gameLossDiameter += gameLossDiameterGrowthRate;
			killBoxes();
			
			if (gameLossDiameter > 933)
			{
				gameState = GameOver;
				
				
				//gameOverTextField.text = "    Game Over\n     Score: " + score + "\nclick to play again";
				//addChild(gameOverTextField);
				addChild(title);
			}
		}
	}
	
	private function onMouseClick(e:MouseEvent):Void
	{
		if (gameState == GameOver)
		{
			score = 0;
			scoreTextField.text = Std.string(score);
			randomizeWeights();
			removeChild(title);
			gameState = Playing;
		}
	}
	
	private function increaseAnxiousness()
	{
		for (anxiousBox in boxes) 
		{
			var anxiousness:Float = 0.0;
			for (otherBox in boxes)
			{
				if (anxiousBox != otherBox)
				{
					var distance:Float = distance(anxiousBox.x, anxiousBox.y, otherBox.x, otherBox.y);
					if (distance <= MaxAnxiousnessDistance)
					{
						// 1 is up close, 0 is max anxiousness distance away
						var distanceFactor = (MaxAnxiousnessDistance - distance) / MaxAnxiousnessDistance;
						
						// 1 is maximum difference, 0 is no difference
						var areaFactor = Math.abs(anxiousBox.AreaFactor() - otherBox.AreaFactor());
						
						// 1 is maximum difference, 0 is no difference
						var degreeOfSquarenessFactor = Math.abs(anxiousBox.UnSquareness() - otherBox.UnSquareness());
						
						// 1 is maximum difference, 0 is no difference
						var colourFactor = Math.abs(anxiousBox.colourScale - otherBox.colourScale);
						// handle the fact that this value wraps around
						if (colourFactor > 3)
						{
							colourFactor = 6 - colourFactor;
						}
						// even though colourScale can go upto 6, the fact that it wraps back to 0 means that the
						// maximum difference can only be 3
						colourFactor /= 3;
												
						anxiousness += distanceFactor * (areaWeight * areaFactor + degreeOfSquarenessWeight * degreeOfSquarenessFactor + colourWeight * colourFactor);
					}
				}
			}
			
			anxiousBox.setAnxiousness(anxiousness);
			anxiousBox.update();
		}
	}
	
	private function killBoxes():Void
	{
		var boxesToRemove = new Array<Box>();
		for (box in boxes)
		{
			if (distance(box.x, box.y, gameLossX, gameLossY) <= gameLossDiameter)
			{
				boxesToRemove.push(box);
			}
		}
		
		for (boxToRemove in boxesToRemove)
		{
			boxToRemove.unload();
			boxes.remove(boxToRemove);
			removeChild(boxToRemove);
			popSound.play();
		}
	}
	
	private function distance(x1:Float, y1:Float, x2:Float, y2:Float):Float {
		return Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));
	}
	
	private function spawnBox(e:TimerEvent)
	{	
		if (gameState == Playing)
		{
			var box = boxFactory.GetRandomBox();
			boxes.push(box);
			addChild(box);
			boxSlideSound.play();
			score++;
			scoreTextField.text = Std.string(score);
		}
	}
	
	static public function main() 
	{
		var stage = Lib.current.stage;
		stage.scaleMode = nme.display.StageScaleMode.NO_SCALE;
		stage.align = nme.display.StageAlign.TOP_LEFT;
		
		Lib.current.addChild(new Main());
	}
	
}
